# Penalized likelihood with the penalty term derived from saddlepoint appx as in 
# Levin (1990) and Platt (1998).
#------------------------------------------------------------------------------#
# 1. Source/load functions.
# The fitting function for the penalized likelihood is HOApenLkhd(), found 
# in source file HOApenLkhd.R.
source("HOApenLkhd.R")
# Compare HOApenLkhd() to saddlepoint methods implemented in the hoa pacakge;
# install.packages("hoa") to install, then
library(hoa)
# Compare to Firth logistic regression applied to the stratified logistic
# model used for matched case control data. This approach has a very similar
# penalty to HOApenLkhd(). Firth logistic regression is implemented in the 
# logistf package; install.packages("logistf") to install, then
library(logistf)
# Compare to conditional logistic regression implemented in the clogit() 
# function in the survival package
library(survival)
# Comare to Firth conditional logistic regression, using the clogitf()
# function that we wrote following G. Heinze's instructions
# and the coxphf package; install coxphf with 
# install.packages("coxphf"), then:
library(coxphf)
source("clogitf.R")
# Lastly, source in a function simMatched() that simulated matched case 
# control data:
source("simMatched.R")
#------------------------------------------------------------------------------#
# 2. Try out our function HOApenLkhd() on simulated data.
# Parameters for simMatched() are the sample size n, coefficient
# of interest beta1 and the number of nuisance covariates p.
# We generate matched sets of 1 case and 4 controls, so n must be
# a multiple of 5. Then number of matched sets is n/5.
testparams = c(50,1,0); names(testparams) = c("n","beta1","p")
set.seed(111)
testdat = simMatched(testparams)
names(testdat)
# Function HOApenLkhd takes a data frame in the format output by 
# simMatched().
HOApenLkhd(testdat) # Estimates only
HOApenLkhd(testdat,drawProfile=TRUE) # Estimates, and draw the penalized profile lkhd
HOApenLkhd(testdat,CI=TRUE) # Estimates, 95% CI and level 5% test of H_0: beta_1=0.
#------------------------------------------------------------------------------#
# 3. Compare output of HOApenLkhd() to other approaches.
# (a) the cond() function from the hoa package. This requires that we 
# first fit a stratified logistic regression with stratum effects for
# the matched sets. To fit with glm() we need a factor variable that
# indicates the strata. Coerce our matched set variable to a factor.
testdat$fm = as.factor(testdat$matchedset)
ff = glm(case~fm+x1,data=testdat,family=binomial())
cond(ff,x1) # hoa estimate is close to est. from HOApenLkhd()
# (b) Firth logistic regression.
logistf(case~fm+x1,data=testdat) # x1 est. slightly smaller 
                                 # than from HOApenLkhd()
# (c) Conditional logistic regression
clogit(case~x1+strata(matchedset),data=testdat) # slightly smaller
# (d) Firth conditional logistic regression
clogitf(case~x1,data=testdat) # smallest estimate so far
#------------------------------------------------------------------------------#
# 4. Repeat comparison on the DES data, with the DES effect of interest adjusted
# for maternal smoking.
DES = read.csv("DES.csv")
summary(DES)
des = data.frame(case=DES$case,matchedset=DES$matched.set,x1=DES$DES,
                 x2=DES$matern.smoke)
HOApenLkhd(des,CI=TRUE) # DES effect estimate 4.37
# Any of the penalized methods (HOApenLkhd, Firth logistic, Firth conditional logistic)
# work, but those that require ML (logistic regression, hoa, conditional logistic regr.)
# do not work.
des$fm = as.factor(des$matchedset)
ff = glm(case~fm+x1+x2,data=des,family=binomial()) # Fails, so can't use hoa
logistf(case~fm+x1+x2,data=des) # Firth estimate of DES is 4.10 
clogit(case~x1+x2+strata(matchedset),data=des) # Conditional LR fails.
clogitf(case~x1+x2,data=des) # Firth cond. logistic estimate is 3.57

# Remove des$fm for what follows.
des$fm = NULL
# Draw profile curves for HOA and Firth. Next part is a real hack:
# Go into the file HOApenLkhd.R and (i) comment out line 86 
# where the Fisher infor submatrix for HOA is extracted, and (ii) 
# uncomment line 87 that uses all the Fisher info as in Firth.
source("HOApenLkhd.R")
firthcurve = HOApenLkhd(des,drawProfile=TRUE,beta1grid=seq(from=0,to=7,length=100))
# Now go back and reset the code to HOA, then source in HOA code
source("HOApenLkhd.R")
pdf(file="HOAvsFirth.pdf")
hoacurve = HOApenLkhd(des,drawProfile=TRUE,beta1grid=seq(from=0,to=7,length=100)) 
abline(v=hoacurve$est[1])
lines(firthcurve$beta1grid,firthcurve$profileLkhd,lty=2)
abline(v=firthcurve$est[1],lty=2)
dev.off()
