# Penalized likelihood with the penalty term derived from saddlepoint appx as in 
# Levin (1990) and Platt (1998).
#------------------------------------------

HOApenLkhd = function(dat,effectLim=10,CI=FALSE, level=0.05,drawProfile=FALSE,
                      beta1grid=seq(from=-5,to=5,length=100)) {
  # - The data frame dat is assumed to include a binary disease status 
  # indicator, a categorical variable matchedset, and covariates. The 
  # covariate of interest is assumed to be the first covariate,
  # followed by any nuisance covariates.
  # - effectLim is a limit on the size of estimated effects. If we
  # find estimates larger than effectLim we'll say the algorithm failed
  # to converge.
  # - level is the level of the hypothesis test that the 
  # parameter of interest is zero. Confidence intervals are (1-level)*100%.
  # - drawProfile: should we draw the penalized profile log-likelihood
  # for the parameter of interest?
  # - beta1grid is the grid of values over which to calculate the penalized
  # profile log-likelihood for drawing
  nbeta = ncol(dat) - 2
  case = dat$case
  # Make a new data frame with case, stratum indicator variables for matched sets,
  # and then the covariates.
  X = getDesignMat(dat)
  # The funtion HOAProfLogLkhd calculates the penalized profile log-likelihood
  # for the regression coefficients, as in eqn (9.17) of the chapter.
  # Maximize this objective function to find the estimates.
  # Use a vector of one's as the initial estimates of the betas.
  oo = optim(rep(1,nbeta),HOAProfLogLkhd,gr=NULL,X=X,case=case,
        method="BFGS",control=list(fnscale=(-1)))
  est = oo$par
  # We have no check for convergence. Instead use a kludge and check 
  # that none of the estimates are huge.
  if(any(abs(est)>effectLim)) {
    stop("Algorithm failed to find estimates less than limit ",effectLim,
         " in absolute value")
  }
  plkhd = oo$value
  if(CI) {
    # Tried to write code to implement penalized likelihood ratio-based
    # CIs, but was not very successful. Works OK when there are no
    # nuisance covariates, but otherwise tends to crash.
    PPL.CI = PPLci(est[1],nbeta,X,case,conf.level=1-level)
    test.rej = !is.in(0,PPL.CI)
  } else {
    PPL.CI = test.rej = NULL
  }
  # Put together output
  out = list(est=est,plkhd=plkhd,ci=PPL.CI,test.rej=test.rej)
  if(drawProfile) { 
    # calculate the profile likelihood for the parameter of 
    # interest, assumed to be the first log-OR, beta1.
    pp = rep(NA,length(beta1grid))
    for(i in 1:length(beta1grid)) {
      print(i)
      pp[i] = profileParInterest(beta1grid[i],nbeta,X,case)
    }
    plot(beta1grid,pp,xlab=expression(beta[1]),ylab="penalized profile lkhd",
         type="l")
    out$beta1grid = beta1grid
    out$profileLkhd = pp
  }
  return(out)
}
#------------------------------------------------
# Worker functions called by HOApenLkhd()
getDesignMat = function(dat) {
  # Construct the design matrix. In hindsight, could have just used
  # R's model.matrix() function to build design matrix, but 
  # for some reason (forgot why) insisted on one dummy variable for
  # each matched set.
  nmatch = nrow(dat)/5 # assume matched sets are 1 case, 4 controls
  fm = as.factor(dat$matchedset)
  # dummy variables as indicators of matched set ID:
  inds = sapply(1:nmatch,FUN=function(i) { as.numeric(fm==i)})
  colnames(inds) = paste0("m",1:ncol(inds))
  covs = as.matrix(dat[,3:ncol(dat)])
  return(cbind(inds,covs))
}
HOAProfLogLkhd = function(beta,X,case) {
  # First find the profile log likelihood for the log odds ratio parameters
  # (betas) by profiling out stratum effects (alpha's)
  alphaInds = (1:(ncol(X)-length(beta))) # stratum effects come first in X
  betaInds = setdiff(1:ncol(X),alphaInds)
  # call a generic profiler() function that maximizes a function, fund,
  # over variables in argument maxInd, holding fixed the variables in 
  # argument fixedInd at values parsFixed. The rest of the arguments 
  # are arguments to func.
  oo = profiler(func=logLkhd,maxInd=alphaInds,fixedInd=betaInds,parsFixed=beta,
                X=X,case=case)
  # oo$par is our alphahat, and oo$value is the profile log-likelihood
  # value at alphahat
  alphahat = oo$par
  pll = oo$value
  # Penalize the profile likelihood by (1/2) det (I_{11}), where
  # I_{11} is the (alpha,alpha) component of the Fisher Info.
  FI = FInfo(c(alphahat,beta),X)
  I11 = FI[alphaInds,alphaInds] # For HOA penalty
  # I11 = FI # KLUDGE: Firth penalty -- uncomment this and comment out above 
  #                    if you want Firth
  return(pll + 0.5*log(det(I11)))
}
logLkhd = function(pars,X,case) {
  return(sum(case*(X%*%pars) - log(1+exp(X%*%pars))))
}
FInfo = function(beta,X) {
  p = as.vector(expit(X%*%beta)); vv = p*(1-p)
  return(t(X)%*%diag(vv)%*%X)
}
expit = function(x) return(exp(x)/(1+exp(x)))
profiler = function(func,maxInd,fixedInd,parsFixed,X,case){
  # - func is a function of p = length(maxInd) + length(fixedInd) parameters.
  # - the indices in maxInd are the parameters to maximize over
  # - the indices in fixedInd are the parameters to hold fixed
  # - parsFixed are the values of parameters to hold fixed
  npars = length(maxInd) + length(fixedInd)
  # Use the optim() function built into R. This requires a function whose
  # first argument is the parameters to be maximized over. Create a
  # version of the input function "func"  whose first argument is the 
  # parameters to be maximized over.
  ofunc = function(parsOptim,X,case) {
    pars = rep(NA,npars)
    pars[maxInd] = parsOptim
    pars[fixedInd] = parsFixed
    return(func(pars,X,case))
  }
  nparOptim = length(maxInd) # number of pars to optimize over
  oo = optim(rep(0,nparOptim),fn=ofunc,X=X,case=case,
             method="BFGS",control=list(fnscale=(-1)))
             # fnscale= -1 says to 
  return(oo)
}

# GOT HERE

PPLci = function(beta1hat,nbeta,X,case,conf.level,tol=1e-5,initHalfWidth=2,maxiter = 10){ 
    # The function profileParInterest profiles out the nuisance covariates
    # from the penalized profile log-likelihood. Use this function to 
    # find a CI for the parameter of interest.
  betaProf.max = profileParInterest(beta1hat,nbeta,X,case)
  crit.val = qchisq(conf.level,df=1)
  objfunc = function(beta1) {
    bpdiff = profileParInterest(beta1,nbeta,X,case)-(betaProf.max-crit.val/2)
    return(bpdiff^2)
  }
  beta1.lower = c(beta1hat-initHalfWidth,beta1hat)
  ci.lower = NA
  liter = 0
  while(is.na(ci.lower) && liter < maxiter) {
    ll = try({ optimize(f=objfunc,interval = beta1.lower) })
    if(class(ll) == "try-error") break()
    if(ll$objective>tol) { 
      beta1.lower[2] = beta1.lower[1]
      beta1.lower[1] = beta1.lower[1]-1
      liter = liter + 1
    } else { 
        ci.lower = ll$minimum 
    }
  }
  beta1.upper = c(beta1hat,beta1hat+initHalfWidth)
  ci.upper = NA
  uiter = 0
  while(is.na(ci.upper) && uiter < maxiter) {
    uu = try({ optimize(f=objfunc,interval = beta1.upper) })
    if(class(uu) == "try-error") break()
    if(uu$objective>tol) { 
      beta1.upper[1] = beta1.upper[2]
      beta1.upper[2] = beta1.upper[2]+1
      uiter = uiter + 1
    } else { 
        ci.upper = uu$minimum 
    }
  }
  if(is.na(ci.lower)) ci.lower = -Inf
  if(is.na(ci.upper)) ci.upper = Inf
  return(c(ci.lower,ci.upper))
}  
profileParInterest = function(beta1,nbeta,X=X,case=case) {
  if(nbeta==1) {
    # then nothing to profile out
    return(HOAProfLogLkhd(beta1,X,case))
  } else {
    nuisanceBetas = (2:nbeta)
    return(profiler(HOAProfLogLkhd,maxInd=nuisanceBetas,fixedInd=1,
                    parsFixed=beta1,X=X,case=case)$value)
  }
}
is.in = function(beta,intl) {
  if(beta>=intl[1] & beta <= intl[2]) { return(TRUE) }
  return(0)
}