# Utility functions for preparing the data for elrm.
#######################################################
# 1. collapseToUniqueX
# A function to prepare unmatched case-control data. The
# function takes arguments
#  - x, a data frame whose rows are the covariate vectors, and
#  - case, a binary vector of case-control status, 
#       with 1=case, 0=control.
# The task is to (i) find the unique covariate vector values
# x_{(j)}, of x and (ii) count the number of subjects, m_j,
# and the number of cases, y_j, with each covariate value 
# x_{(j)}. As output, the function combines the m_j's and
# y_j's into vectors m and y, respectively, and stacks the
# x_{(j)}'s as rows of a matrix. Then the function column
# binds all three together and returns them in a data frame.
collapseToUniqueX = function(x,case) {
  out.x = out.y = out.m = NULL # initialize outputs
  # Easy to get the unique rows of x with unique():
  out.x = unique(x) 
  # To get m and y do a loop through the unique covariate 
  # vector values, x_{(j)} and count:
  #  m_j - the number of subjects with x_{(j)}
  #  y_j - the number of cases with x_{(j)}
  q = nrow(out.x) 
  m = y = rep(0,q) # initialize m and y
  for(j in 1:q){
    # Use the function findMatches defined below to find
    # indices of rows in x that match x_{(j)} (out.x[j,]).
    matches = findMatches(out.x[j,],x)
    # The length of matches is m_j and the number
    # of cases in matches is y_j
    m[j] = length(matches)
    y[j]=sum(case[matches])
  } 
  return(data.frame(m=m,y=y,out.x))
}

findMatches = function(v,m) {
  # find row-indices of matrix m that match vector v
  matches = NULL # initialize
  for(i in 1:nrow(m)) {
    if(all(v==m[i,])) matches = c(matches,i)
  }
  return(matches)
}

#######################################################
# 2. collapseToUniqueXStrat
# A function to prepare matched case-control data. Call
# collapseToUniqueXStrat on each matched set and combine
# the results.
collapseToUniqueXStrat = function(x,case,strata) {
  I = max(strata) # number of matched sets
  out = NULL # will be output data frame
  for(i in 1:I) {
    temout = collapseToUniqueX(x[strata==i,],case[strata==i])
    temout = data.frame(strata = rep(i,nrow(temout)),temout)
    out = rbind(out,temout)
  }
  # The stratum variable needs to be a factor so that
  # when passed to elrm in a formula, R will 
  # convert the factor into stratum effects.
  out$strata = factor(out$strata)
  return(out)
}
# Example call
# x = DES[,c("DES","matern.smoke")]
# out = collapseToUniqueXStrat(x=x,case=DES$case,strata=DES$matched.set)