#################################################################
## Conditional distribution for exact inference
#################################################################
# Notation:
# The coefficient of the variable x1 is the parameter of interest.
# The coefficient of the variable x2 is a nuisance parameter.
# The sufficient statistic for the parameter of interest is
# t_{beta_1}=sum(d * x1), where d is a binary disease-status indicator.
# The sufficient statistic for the nuisance parameter is
# t_{beta_2} = sum(d * x2). Shorten to t2.
# In what follows, shorten t_{beta_1} to t1 and t_{beta_2} to t2.
# 
# Overview of approach to finding the conditional pmf:
# The support of the conditional pmf is the set of t1-values 
# that can be obtained as sum(tilde{d}*x1) for some binary disease-status 
# vector tilde{d}. When there are I matched-sets, tilde{d} is comprised  
# of I sub-vectors, each of length equal to the number of subjects in 
# matched-set i. Let M_i be the number of controls in matched-set i.
# The subvector of tilde{d} corresponding to matched-set i consists of 
# a single one and M_i zeros.  In addition to the support, we need the 
# counts, C(tilde{t1}), of the number of tilde{d} that give 
# sum(tilde{d}*x1)=tilde{t1}. Can get the pmf by enumeration of all 
# possible tilde{d} vectors, keeping those that satisfy the conditioning 
# constraint sum(tilde{d}*x2)=t2. For I matched-sets, there are 
# prod_{i=1}^I (M_i+1) possible tilde{d} vectors.
#
# Picture all possible tilde{d} vectors as leaves of a tree with
# internal nodes. At the root node is a null vector. The root splits 
# into (M_1+1) nodes and each is assigned one of the M_1+1 possible 
# sub-vectors tilde{d}^1 for matched set 1, consisting of a single 1 
# and M_1 zeros: (1,0,...,0), (0,1,0,...,0), ..., (0,...,0,1).
# At each of the (M_1+1) nodes we split (M_2+1) times, and
# extend tilde{d}^1 by one of the (M_2+1) possible sub-vectors,
# tilde{d}^2, for matched set 2. Continue until we reach the 
# leaves at matched set I, where we will have a complete tilde{d}. 
#
# Recursively construct the above the tree and at each leaf, 
# tilde{d}, test whether sum(tilde{d}*x2) = t2. If so
# record the value of tilde{t1}=sum(tilde{d}*x1).
# After recording all the tilde{t1}'s, we can tabulate
# them to get the support (unique values) and counts.
#
# Structure of the recursive function: 
#  1. See if we are an internal node of the tree. If so, split (M_i+1)
#     times; i.e., call self (M_i+1) times with dprev extended by the
#     the possible sub-vectors for matched set i.
#  2. Else we are at a leaf, so test whether sum(tilde{d}*x2)=t2. 
#     If so, record the value tilde{t1}=sum(tilde{d}*x1).
# 
# Use a wrapper function to (i) create a data structure which 
# bundles together all the data the recursive function will need, 
# (ii) make the initial call to the recursive function to enumerate 
# the tilde{d}'s and record the tilde{t1}'s, and (iii) call a summary 
# function to find the unique values of tilde{t1} and count the number 
# of copies of each.
condDistT1 = function(x1,x2,d,matched.sets) { # Wrapper function.
  # (i) Bundle the data that the recursive function will need
  I = max(matched.sets)
  t2 = sum(x2*d)
  dat = list(x1=x1,x2=x2,matched.sets=matched.sets,I=I,t2=t2)
  # (ii) Call recursive function condDistT1Suppt, defined below
  tt1 = condDistT1Suppt(dprev=NULL,i=1,dat=dat,tt1=NULL)
  # (iii) Summarize as a pmf using cdT1.summary(). Summary
  # function is defined below definition of condDistT1Suppt
  out = cdT1.summary(tt1)
  return(out)
}

condDistT1Suppt = function(dprev,i,dat,tt1) { 
  # Recursive function for the i-th matched-set.
  # - dprev is the tilde{d} vector from previous matched-sets 1,...,i-1.
  # - i is the current matched-set.
  # - dat is a list with all the data needed by this function.
  if(i <= dat$I)  { # Internal node; call self M_i+1 times
    Mi1 = sum(dat$matched.sets==i) #Variable to hold M_i+1
    #Create the possible subvectors for i-th matched-set, as rows of matrix.
    vecs = diag(1,nrow=Mi1,ncol=Mi1) 
    for(j in 1:Mi1) {
      # Concatenate dprev with one of the sub-vectors for the current
      # matched set and call self.
      dcur = c(dprev,vecs[j,]) 
      tt1 = condDistT1Suppt(dcur,i+1,dat,tt1)
    }
  } else { # We're at a leaf
    tilde.d = dprev
    if(sum(tilde.d*dat$x2)==dat$t2) {
      new.tt1=sum(dat$x1*tilde.d) # another tilde{t1} 
      # Append new.tt1 to the tt1 vector.
      return(c(tt1,new.tt1))
    } else {
      return(tt1)
    }
  }
  return(tt1)
}

cdT1.summary = function(cd) { 
  # Summarize the vector of t1-values generated by recursive function.
  tem = table(cd)
  cd.suppt = as.numeric(names(tem))
  cd.counts = as.numeric(tem)
  cd.freq = cd.counts/length(cd) # This is the null distribution
  return(list(suppt=cd.suppt,counts=cd.counts,freq=cd.freq))
}
# Example call to condDistT1: 
#  cd.DES = condDistT1(x1=DES$DES,x2=DES$matern.smoke,d=DES$case,
#                      matched.sets=DES$matched.set)


#################################################################
# GOT HERE
#################################################################


#################################################################
## Conditional distribution for approximate exact inference
#################################################################
# Notation: Similar to above, but now we want the distribution of 
# y given t_{beta_2} and t_{bold g}. 

# reduce data to q unique x's and corresp. y's within each matched set

# The coefficient of the variable x1 is the parameter of interest.
# The coefficient of the variable x2 is a nuisance parameter.
# The sufficient statistic for the parameter of interest is
# t1=sum(tilde{d} * x1), where tilde{d} is a binary disease-status
# indicator such that there is 1 one and 4 zeros in each sub-vector
# corresponding to a matched set.
# The sufficient statistic for the nuisance parameter is
# t2 = sum(tilde{d} * x2).

condDistYSuppt = function(yprev,i,I) {
  if(i <= I){ # haven't hit leaves yet, so call self q times
    q = sum(stratum==i) # number of unique x's in matched set i
    vecs = diag(1,nrow=q,ncol=q) # possible sub-vectors, as a matrix
    for(j in 1:q) {
      # concatenate yprev with a subvector, to get a
      # y vector that includes matched sets up to i.
      ycur = c(yprev,vecs[j,]) 
      condDistYSuppt(ycur,i+1,I)
    }
  } else { # We're at a leaf, so test if y yields t2 and, if so, add to global list of y's
    tilde.y = yprev
    if(sum(x2*tilde.y)==t2) {
      # Add this y to the global variable ty
      assign("ty",rbind(ty,tilde.y),envir=.GlobalEnv) 
    }
  } 
}