# We call our implementation of Zhang's modified score approach
# logistzCC, because it is like Heinze's logistf (Firth logistic 
# regression), but specifically for case-control data. We 
# use Zhang's notation throughout: theta = (alpha,beta), where
# alpha is Zhang's intercept and beta are the log-odds ratios. NB:
# the alpha^* in the book chapter is log(rho) + alpha, where rho=n1/n0.
## ------------------------------------------------------------------------
logistzCC = function(formula, data, init=NULL,CI=FALSE,
                     conf.level=0.95,verbose.output=FALSE) {
  # Input:
  # - formula is the model formula
  # - data is a data frame that includes the variables named in formula
  # - conf.level is the level for confidence intervals
  # - init (optional) is the initial regression parameter values to use
  # - verbose.output: Should we return the output of the root-finding
  #   algorithm?
  # logistzCC (1) sets up the data needed to compute the estimator, 
  # (2) defines the modified score function whose root defines the 
  # modified score estimator, (3) calls a root-finding function to
  # find the root, (4) computes a standard error using
  # Zhang's "Remark 3", and (5) computes a PPL confidence interval.
  # 1. Set up the data that modscore will need. 
  mf = model.frame(formula,data) # a data structure from which we extract
  D = model.response(mf)         # the response variable
  X = model.matrix(formula,data) # includes a column for the intercept
  n=length(D); n1 = sum(D==1); n0 = sum(D==0); rho = n1/n0
  # 2. Define modscore() within logistzCC().
  modscore = function(theta) { 
    # The components of the modified profile score depend on the parameters,
    # theta, only through the following variable pi.
    pi = expit(log(rho) + as.numeric(X%*%theta))
    uu = U(pi,D,X)
    mu.t = mu.tilde(pi,X)
    D.t = D.tilde(pi,X,n1)
    mod  = 0.5*(mu.t + (1-rho)*D.t) # Zhangs's first modified 
                                    # score function U_{M1}
    return(uu-mod)
  }
  # 3. Call the root finder nleqslv() on modscore(), with initial value
  # init if passed by the user, or a vector of zeros if not.
  require(nleqslv) 
  if(is.null(init)) {init = rep(0,ncol(X))}
  root.info = nleqslv(init,modscore)
  estimate = root.info$x; names(estimate) = colnames(X)
  # 4. Covariance and SEs; these are a function of the estimated parameter values.
  pi.final = expit(log(rho) + as.numeric(X%*%estimate))
  var = logistVar(pi.final,X)
  ses = sqrt(diag(var))
  out=list(coefficients=estimate,ses=ses,var=var)
  # 5. PPL CI
  if(CI) {
    if(ncol(X)==2) {
      PPL.CI = PPLci(estimate[2],X,D,conf.level)
    } else {
      stop("Model has ",ncol(X)-1," covariates. Only 1 allowed for PPL CIs.")
    }
  out$PPL.CI=PPL.CI
  }
  if(verbose.output) out$root.info = root.info
  return(out)
}
## ------------------------------------------------------------------------
# Functions needed by logisfCC
## ------------------------------------------------------------------------
expit = function(t) { return(exp(t)/(1+exp(t)))}
## ------------------------------------------------------------------------
U = function(pi,d,X) {
  return(colSums((d-pi)*X))
}
## ------------------------------------------------------------------------
D.tilde = function(pi,X,n1) {
  return(colSums(pi*(1-pi)*X)/n1)
}
## ------------------------------------------------------------------------
mu.tilde = function(pi,X) {
  pp1 = ncol(X) # p+1
  n = nrow(X)
  s.inv = solve(S.tilde(pi,X))
  out = rep(0,pp1)
  for(m in 1:pp1) {
    out[m] = tr(Lambda.tilde(pi,X,m)%*%s.inv)
  }
  return(out)
}
tr = function(mat) { return(sum(diag(mat)))}
## ------------------------------------------------------------------------
Lambda.tilde = function(pi,X,m){
  pp1 = ncol(X) # p+1
  n = nrow(X)
  out = matrix(0,nrow=pp1,ncol=pp1)
  for(i in 1:n) {
    H.im = X[i,]%*%t(X[i,])*X[i,m]
    out = out - pi[i]*(1-2*pi[i])*(1-pi[i])*H.im
  }
  return(out/n)
}
## ------------------------------------------------------------------------
S.tilde = function(pi,X) {
  # Can show that Zhang's S-tilde is  the negative Hessian of the 
  # profile log-likelihood, which is 
  #     (1/n) sum_i X_i^T X_i * pi_i*(1-pi_i),
  # and that the sum can be written in matrix form as X^T diag(pi(1-pi)) X.
  n = length(pi)
  ss = t(X)%*%diag(pi*(1-pi))%*%X
  return(ss/n)
}
logistVar= function(pi.final,X) {
  # The interpretation of Zhang's Remarks 3 and 1 is that the 
  # variance of *both* modified score estimators is the inverse 
  # of (-1* Hessian) evaluated at the estimate, and that 
  # -1*Hessian is n *S.tilde in Zhang's notation.
  # Heinze and Schemper suggest that the same variance estimate
  # applies to Firth logistic regression.
  n = nrow(X)
  return(solve(n* S.tilde(pi.final,X))) 
}
#-----------------------------------------------------------------
# PPL confidence intervals: Based on inverting a penalized profile
# likelihood ratio statistic. The penalized profile likelihood is
# obtained by integrating Zhang's modified score equation.
# The log of this penalized profile likelihood (PPL) is a function
# of Zhang's theta = (alpha,beta). We need to profile out alpha
# to obtain a PPL of one variable, PPL(beta). Can use
# the "drop down 2" method of finding a 95% CI from PPL(beta).
# More generally we drop down by crit.val/2 where crit.val is 
# the conf.level critical value from a chi-squared with 1 d.f.
PPLci = function(betahat,X,D,conf.level){ 
  PPLbeta.max = PPLbeta(betahat,X,D) # max of PPL curve
  drop.down = qchisq(conf.level,df=1)/2 # amount to drop from max
  # Solve for limits of confidence interval. This amounts to searching 
  # for beta values such that the PPL equals PPLbeta.max - drop.down.
  PPL.objective = PPLbeta.max - drop.down
  # This is a root-finding problem, but we need to be able to constrain the 
  # search for the lower CI limit to beta < betahat and the search for the 
  # upper CI limit to beta > betahat. We don't know of a root finding
  # algorithm in R that allows constraints. 
  # Instead, use optimize() to find the minimum value of 
  #     (PPLbeta(beta) - PPL.objective)^2
  # constrained to be either greater than or less than betahat.
  objfunc = function(beta) {
    return((PPLbeta(beta,X,D)-PPL.objective)^2)
  }
  BIGNUM = 15
  ci.lower = optimize(f=objfunc,interval = c(-BIGNUM,betahat))$minimum
  ci.upper = optimize(f=objfunc,interval = c(betahat,BIGNUM))$minimum
  # format CI as a matrix to print nicely
  ci = matrix(c(ci.lower,ci.upper),nrow=1,ncol=2) 
  colnames(ci) = c("CI.Lower","CI.Upper")
  rownames(ci) = names(betahat)
  return(ci)
}
PPLbeta = function(beta,X,D) {
  # For fixed beta maximize logPPL over alphas.
  # optimize() needs an interval of alphass to search over. 
  # Don't have a good idea what such an interval should be, so
  # just take a guess that (-100,100) is a big enough interval.
  alpha.lim = c(-100,100)
  oo = optimize(f=logPPL.ZhangH,interval=alpha.lim,beta=beta,X=X,D=D,
                maximum=TRUE)
  return(oo$objective)
} 
# logPPL.Zhang:
# Integrating Zhang's Hessian-based modified score equation gives
# a penalized profile likelihood equal to: 
#   the log profile likelihood plus
#   the log of the Hessian-based Jeffreys penalty minus 
#   (1/2*n1) sum_{i=1} pi(alpha,beta,x_i),
# where p(alpha,beta,x) = exp(log(rho)+alpha+beta*x)/[1+exp(log(rho)+alpha+beta*x)],
# alpha is the intercept and beta is the log OR coefficient of the 
# one covariate. The covariate x is the second column of the design
# matrix X.
logPPL.ZhangH = function(alpha,beta,X,D) {
  n1 = sum(D); n0 = sum(1-D); rho = n1/n0
  expterm = as.numeric(log(rho) + alpha + beta*X[,2])
  p = expit(expterm)
  return(logPL(expterm,D)+log.JeffH(p,X)-(1-rho)*sum(p)/(2*n1))
}
# The log profile likelihood is shown by Zhang to have
# the same form as the likelihood for logistic regression based on 
# prospective data.
logPL = function(expterm,D) {
  return(sum(D*(expterm) - log(1+exp(expterm))))
}
log.JeffH = function(p,X) {
  # Can show that the Hessian is X^T W X where X is the design matrix
  # (including a column of 1's for the intercept) and W is a diagonal matrix
  # of variance terms p*(1-p).
  vv = p*(1-p)
  Hess = t(X)%*% diag(vv) %*% X
  return(0.5*log(det(-1*Hess)))
}
## ------------------------------------------------------------------------
# Testing
## ------------------------------------------------------------------------
# Simulate some data
#simdat = function(n0,n1,beta) {
#  # Dist'n of covariate x in controls is standard normal.
#  # Can show it is normal with mean beta and sd 1 in cases.
#  case = c(rep(1,n1),rep(0,n0))
#  x = c(rnorm(n1,mean=beta,sd=1),rnorm(n0,mean=0,sd=1))
#  return(data.frame(case=case,x=x))
#}
#beta = 3
#n1=10; n0=40
#set.seed(123)
#dat = simdat(n0,n1,beta)
#names(dat) # response is called "case", covariate is called "x"
#form = formula(case ~ x)
#logistzCC(form,dat) 
#logistzCC(form,dat,CI=TRUE) 
