# We call our implementation of Zhang's modified score approach
# logistzCC, because it is like Heinze's logistf (Firth logistic 
# regression), but specifically for case-control data.
## ------------------------------------------------------------------------
logistzCC = function(formula, data, init=NULL,
                     conf.level=0.95,verbose.output=FALSE) {
  # Input:
  # - formula is the model formula
  # - data is a data frame that includes the variables named in formula
  # - conf.level is the level for confidence intervals
  # - init (optional) is the initial regression parameter values to use
  # - verbose.output: Should we return the output of the root-finding
  #   algorithm?
  # logistzCC (1) sets up the data needed to compute the estimator, 
  # (2) defines the modified score function whose root defines the 
  # modified score estimator, (3) calls a root-finding function to
  # find the root, (4) computes a standard error using
  # Zhang's "Remark 3", and (5) computes a PPL confidence interval.
  # 1. Set up the data that modscore will need. 
  mf = model.frame(formula,data)
  D = model.response(mf) # response variable
  X = model.matrix(formula,data) # includes a column for the intercept
  n=length(D); n1 = sum(D==1); n0 = sum(D==0); rho = n1/n0
  # 2. Define modscore() within logistzCC().
  modscore = function(theta) { 
    pi = expit(log(rho) + as.numeric(X%*%theta))
    uu = U(pi,D,X)
    mu.t = mu.tilde(pi,X)
    D.t = D.tilde(pi,X,n1)
    mod  = 0.5*(mu.t + (1-rho)*D.t) # Zhangs's first modified 
                                    # score function U_{M1}
    return(uu-mod)
  }
  # 3. Call the root finder nleqslv() on modscore(), with initial value
  # init if passed by the user, or a vector of zeros if not.
  require(nleqslv) 
  if(is.null(init)) {init = rep(0,ncol(X))}
  root.info = nleqslv(init,modscore)
  estimate = root.info$x; names(estimate) = colnames(X)
  # 4. Covariance, SEs and Wald CI
  pi.final = expit(log(rho) + as.numeric(X%*%estimate))
  var = logistVar(pi.final,X)
  ses = sqrt(diag(var))
  Wald.CI = Waldci(estimate,ses,conf.level)
  # 5. PPL CI
  if(ncol(X)==2) {
    PPL.CI = PPLci(logPPL.ZhangH,estimate[2],X,D,conf.level)
  } else {
    stop("Model has ",ncol(X)-1," covariates. Only 1 allowed for PPL CIs.")
  }
  colnames(PPL.CI) = colnames(Wald.CI)
  rownames(PPL.CI) = rownames(Wald.CI)
  # Return the results.
  out=list(coefficients=estimate,Wald.CI=Wald.CI,PPL.CI = PPL.CI,var=var)
  if(verbose.output) out$root.info = root.info
  return(out)
}
## ------------------------------------------------------------------------
# Functions needed by logisfCC
## ------------------------------------------------------------------------
expit = function(t) { return(exp(t)/(1+exp(t)))}
## ------------------------------------------------------------------------
U = function(pi,d,X) {
  return(colSums((d-pi)*X))
}
## ------------------------------------------------------------------------
D.tilde = function(pi,X,n1) {
  return(colSums(pi*(1-pi)*X)/n1)
}
## ------------------------------------------------------------------------
mu.tilde = function(pi,X) {
  pp1 = ncol(X) # p+1
  n = nrow(X)
  s.inv = solve(S.tilde(pi,X))
  out = rep(0,pp1)
  for(m in 1:pp1) {
    out[m] = tr(Lambda.tilde(pi,X,m)%*%s.inv)
  }
  return(out)
}
tr = function(mat) { return(sum(diag(mat)))}
## ------------------------------------------------------------------------
Lambda.tilde = function(pi,X,m){
  pp1 = ncol(X) # p+1
  n = nrow(X)
  out = matrix(0,nrow=pp1,ncol=pp1)
  for(i in 1:n) {
    H.im = X[i,]%*%t(X[i,])*X[i,m]
    out = out - pi[i]*(1-2*pi[i])*(1-pi[i])*H.im
  }
  return(out/n)
}
## ------------------------------------------------------------------------
S.tilde = function(pi,X) {
  pp1 = ncol(X) # p+1
  n = nrow(X)
  out = matrix(0,nrow=pp1,ncol=pp1)
  for(i in 1:n) {
    H.i1 = X[i,]%*%t(X[i,])
    out = out + pi[i]*(1-pi[i])*H.i1
  }
  return(out/n)
}
logistVar= function(pi.final,X) {
  # My interpretation of Zhang's Remarks 3 and 1 is that the 
  # variance of *both* modified score estimators is the inverse 
  # of (-1* Hessian) evaluated at the estimate, and that 
  # -1*Hessian is n *S.tilde in Zhang's notation.
  # Heinze and Schemper suggest that the same variance estimate
  # applies to Firth logistic regression.
  n = nrow(X)
  return(solve(n* S.tilde(pi.final,X))) 
}
Waldci = function(estimate,ses,conf.level) {
  crit.val = qnorm((1-conf.level)/2,lower.tail=FALSE)
  Wald.CI = cbind(estimate-1.96*ses,estimate+1.96*ses)
  colnames(Wald.CI) = c("CI.Lower","CI.Upper")
  rownames(Wald.CI) = names(estimate)
  Wald.CI = Wald.CI[-1,,drop=FALSE] # CI for intercept not meaningful (?)
  return(Wald.CI)
}
## ------------------------------------------------------------------------
# PPL confidence intervals:
# An alternative to solving the Zhang H modified score equation is to
# integrate the  modified score and maximize the resulting function,
# which we can think of as a penalized profile likelihood (PPL). 
# Instead of the PPL we work with the log PPL, logPPL.ZhangH, which
# is a function of the intercept alpha and the log OR beta. We assume
# a single covariate.
# logPPL.ZhangH can be used to get confidence intervals for beta. First we need
# to eliminate alpha by profiling *again*; that is, for a grid of beta
# values, maximize logPPL.ZhangH(alpha,beta)
# to get a function of beta. Call this function betaProf.ZhangH(beta).
# Can use the "drop down 2" method of finding a 95% CI from betaProf.ZhangH(beta);
# that is, the CI is all beta's such that 
# betaProf.ZhangH(beta) >= betaProf.ZhangH(betahat) - 2, where
# betahat is the estimate (maximizer). The "2" in "drop down 2" is 
# approximately half the 0.95 critical value from a chi-squared on 1 d.f. 
# More generally we can replace 2 with crit.val/2 where crit.val is 
# the conf.level critical value from a chi-squared with 1 d.f.
# Coding note:
# We will use the same general approach outlined above to get 
# confidence intervals for other logPPLs, so the functions are
# written generically, and take the particular logPPL function as
# an argument.
PPLci = function(logPPLfunc,betahat,X,D,conf.level,alpha.lim=c(-100,100)){ 
  # Solve for limits of confidence interval. This amounts to searching 
  # for beta values such that the penalized profile likelihood for beta
  # equals betaProf.max - crit.val/2, where betaProf.max is the max
  # value of the penalized profile likelihood for beta and crit.val
  # is the conf.level critical value from a chi-squared on 1 d.f.
  # Would have liked to do this search as a root-finding problem using
  # our root finder nleqslv(), but we need to be able to constrain the 
  # search for the lower CI limit to beta < betahat and the search for the 
  # upper CI limit to beta > betahat, and couldn't see how to do this.
  # Instead, use optimize() to find the minimum value of 
  #  (betaProf(beta) - (betaProf.max-crit.val/2))^2.
  # Inputs:
  # - logPPLfunc is the logPPL function to use (currently only logPPL.ZhangH)
  # - betahat is the maximum PPL estimate of the log OR beta
  # - X is the design matrix
  # - D is the vector of disease status
  # - conf.level is the confidence level for the interval estimator
  # Output:
  # - the PPL confidence interval
  beta.lower = c(-15,betahat); beta.upper = c(betahat,15) 
  betaProf.max = betaProf(betahat,X,D,alpha.lim,logPPLfunc)
  crit.val = qchisq(conf.level,df=1)
  objfunc = function(beta) {
    bpdiff = betaProf(beta,X,D,alpha.lim,logPPLfunc)-(betaProf.max-crit.val/2)
    return(bpdiff^2)
  }
  ci.lower = optimize(f=objfunc,interval = beta.lower)$minimum
  ci.upper = optimize(f=objfunc,interval = beta.upper)$minimum
  # format CI as a matrix to match formatting of Wald CI
  ci = matrix(c(ci.lower,ci.upper),nrow=1,ncol=2) 
  return(ci)
}
betaProf = function(beta,X,D,alpha.lim=c(-100,100),logPPLfunc) {
  # for fixed beta maximize logPPLfunc over alphas in alpha.lim
  oo = optimize(f=logPPLfunc,interval=alpha.lim,beta=beta,X=X,D=D,
                maximum=TRUE)
  return(oo$objective)
} 
# logPPL.Zhang:
# Integrating Zhang's Hessian-based modified score equation gives
# a penalized profile likelihood equal to: 
#   the log profile likelihood plus
#   the log of the Hessian-based Jeffreys penalty minus 
#   (1/2*n1) sum_{i=1} p(alpha,beta,x_i),
# where p(alpha,beta,x) = exp(alpha+beta*x)/[1+exp(alpha+beta*x)],
# alpha is the intercept and beta is the log OR coefficient of the 
# one covariate. If we set coef = (alpha,beta) and let X be the design 
# matrix, with a column of ones for the intercept, then 
# X%*%coef = alpha+beta*x. The coefficient alpha has to be the first 
# argument to the function for the call to optimize in betaProf.ZhangH.
logPPL.ZhangH = function(alpha,beta,X,D) {
  coef = c(alpha,beta)
  p = expit(as.numeric(X%*%coef))
  n1 = sum(D)
  return(logPL(coef,X,D)+log.JeffH(p,X)-sum(p)/(2*n1))
}
# The log profile likelihood is shown by Zhang to have
# the same form as the likelihood for logistic regression based on 
# prospective data.
logPL = function(coef,X,D) {
  return(sum(D*(as.numeric(X%*%coef)) - log(1+exp(as.numeric(X%*%coef)))))
}
log.JeffH = function(p,X) {
  # Can show that the Hessian is X^T W X where W is a diagonal matrix
  # of variance terms p*(1-p).
  vv = p*(1-p)
  Hess = t(X)%*% diag(vv) %*% X
  return(0.5*log(det(-1*Hess)))
}
## ------------------------------------------------------------------------
# Testing
## ------------------------------------------------------------------------
# Simulate some data
simdat = function(n0,n1,beta) {
  # Dist'n of covariate x in controls is standard normal.
  # Can show it is normal with mean beta and sd 1 in cases.
  case = c(rep(1,n1),rep(0,n0))
  x = c(rnorm(n1,mean=beta,sd=1),rnorm(n0,mean=0,sd=1))
  return(data.frame(case=case,x=x))
}
beta = 3
n1=10; n0=40
set.seed(123)
dat = simdat(n0,n1,beta)
names(dat) # response is called "case", covariate is called "x"
form = formula(case ~ x)
logistzCC(form,dat) 