# Approximate exact inference using elrm.
# Clear workspace before starting
rm(list=ls())

# Load the package:
library(elrm)

# Read in DES data
DES = read.csv("Data/DES.csv")
summary(DES)

#################################################################
# For what follows, the variable of interest is 
# x1 = DES and the variable x2=matern.smoke is a nuisance.
# We will do two analyses of the DES data. In the first,
# we ignore the matching to illustrate inference for 
# unmatched data. In the second analysis, we account 
# for the matching.

x = DES[,c("DES","matern.smoke")]
case = DES$case

#############################################################
# 1. Unmatched analysis
#############################################################
# Prepare the data:
# Before calling elrm we need to find the unique 
# covariate vector values, x_{(j)}, in the data matrix x.
# Then for each x_{(j)}, count the number of subjects, 
# m_j, and the number of cases, y_j, having x_{(j)}.
# Use the utility function collapseToUniqueX() in the 
# source file collapse.R:
source("Functions/collapse.R")
# collapseToUniqueX takes the following arguments:
# - x: the covariate data
# - case: case/control status, coded as 1=case,0=control
# and returns a data frame that combines the m_j's, y_j's
# and x_{(j)}'s.
dat = collapseToUniqueX(x,case)
dat
#############################################################
# Specify the logistic model and parameters of interest:
# Elrm expects these as formula objects. The specification
# of the response in the formula for the logistic model 
# is a bit non-standard: elrm expects y/m a response of 
# the form y/m, for vectors y and m, where y_j is the 
# number of  cases and m_j is the total number with unique
# covariate value x_{(j)}; j = 1,...,q.
logist.model = formula(y/m ~ DES + matern.smoke)
# specify the DES effect as the parameter of interest
terms.of.interest = formula(~ DES)
# The default number of MCMC replicates in elrm is 
# set very low. Instead, we'll use 1 million replicates.
MCMC.iter = 1000000 
#############################################################
out.unmatched = elrm(logist.model,terms.of.interest,
                     iter = MCMC.iter,dataset=dat)
summary(out.unmatched)


############################################################
# 2. Matched analysis
#############################################################
# With matched data we need to prepare the data separately
# within each matched set by calling collapseToUniqueX
# separately on each matched set. The function 
# collapseToUniqueXStrat in the source file collapse.R 
# will do this and takes the following arguments:
# - x: the covariate data
# - case: case/control status, coded as 1=case,0=control
# - strata: matched set identifiers from 1,...,I where I
#           is the number of matched sets.
strata = DES$matched.set
dat = collapseToUniqueXStrat(x,case,strata)
dat

# Specify a stratified logistic model.
logist.model = formula(y/m ~ strata + DES + matern.smoke)
# Specify the DES effect as the parameter of interest.
terms.of.interest = formula(~ DES)
MCMC.iter = 1000000
out = elrm(formula=logist.model, interest = terms.of.interest,
           iter=MCMC.iter, dataset=dat)
# Takes much longer for matched than for unmatched data, 
# (about 2 mins vs 1/2 min) because we're conditioning on more: 
# the number of cases per stratum, rather than in total.
summary(out) 
