# Clear workspace before starting
rm(list=ls())

# Read in DES data
DES = read.csv("Data/DES.csv")
summary(DES)

#################################################################
## Hypothesis testing
# For most of what follows, the variable of interest is 
# x1 = DES and x2=matern.smoke is a nuisance variable.
#
# Find the conditional distribution of the sufficient statistic
# for the DES effect (T_{beta_1}), given the sufficient statistic for 
# matern.smoke (T_{beta_2}) and the covariate distributions (T_{bold g}).
# In this file we shorten T_{beta_1} to T1 and T_{beta_2} to T2.
# Use the function condDistT1() from the source file condDist.R.
# This function takes the data on x1, x2, disease status, d, and 
# the matched set indicators for each subject and returns a list 
# with components: 
#  $suppt - the support points tilde{t1}
#  $counts - the counts C(tilde{t1}) of the number
#    of binary sequences tilde{d} that satisfy the conditioning
#    constraints and yield sufficient statistic value tilde{t1}
#  $freq - For each tilde{t}_{beta_1}, the count C(tilde{t1}) 
#    divided by the sum over all tilde{t1}. This is the 
#    conditional pmf under the null hypothesis beta_1 = 0.
source("Functions/condDist.R")
cd.DES = condDistT1(x1=DES$DES,x2=DES$matern.smoke,d=DES$case,
                    matched.sets=DES$matched.set) # takes about 30 sec.
cd.DES

# plot null distribution
pdf(file="Figures/DEScond0.pdf")
par(ps=18)
barplot(cd.DES$freq,names.arg=cd.DES$suppt)
#title(main="Exact conditional null distribution for \nDES given maternal smoking and beta=0")
dev.off()

# Conditional scores statistic function. The argument cd is 
# a conditional distribution returned by condDistT1().
condScore = function(cd) {
  mu = sum(cd$freq*cd$suppt)
  sig2 = sum(cd$freq*(cd$suppt-mu)^2)
  scores = (cd$suppt-mu)^2/sig2
  return(scores)
}
DESscore = condScore(cd.DES)
# Generate a LaTeX table of conditional probabilities and conditional scores
# over the support of the conditional distribution.
tem = rbind(cd.DES$suppt,cd.DES$freq,DESscore)
rownames(tem) = c("support", "cond. prob.","cond. score")
library(xtable)
xtable(tem,colnames=FALSE,digits=3)

#################################################################
### Estimation
## Confidence regions obtained by inverting the conditional
# probabilities test: Show (i) the p-value curve, as a function of the
# null hypothesis beta_1^0, so that we can read off a 95% conf. region 
# for the DES example, and (ii) that the region might not be an interval.
#
#Set alpha = 0.05 for level (1-alpha) = 0.95 confidence regions.
alpha = 0.05
#
# Inverting the test: Need to calculate p-values for tests 
# of H0:beta_1=beta_1^0 for a range of beta_1^0 values. (Below,
# we'll use a grid of 1000 values between 1 and 3).
# The arguments to the following pval() function are
# - cd, a conditional distribution object output by condDistT1,
# - t1, the observed value of the sufficient statistic for
#     the parameter of interest, and
# - beta10, the hypothesized value of beta_1 for which we want a p-value.
pval = function(cd,t1,beta10) {  # conditional probability test p-value
  # Need the pmf under beta_1^0, which has numerator terms 
  #  C(tilde{t1})exp(tilde{t1}*beta_1^0). The denominator is  
  #  the sum of numerator terms over all tilde{t1}.
  num = cd$counts*exp(cd$suppt*beta10)
  pmf = num/sum(num)
  ind = (cd$suppt == t1)
  # the p-value is the sum over support points with probability as 
  # small or smaller than the probility of the observed value t1.
  p = sum(pmf[pmf<=pmf[ind]]) 
  return(p)
}
# Set up a grid of 1000 beta_1^0's from 1 to 3 and get p-values.
# The sufficient statistic, t1, for the DES effect beta_1 is the 
# number of cases with DES.
t1=sum(DES$case*DES$DES) #observed value of t1 is 7
gridlen = 1000
betas = seq(from=1,to=3,length=gridlen)
pp = rep(NA,gridlen) 
for(i in 1:gridlen) {
  pp[i] = pval(cd.DES,t1,beta10=betas[i])
}
# Look at a level (1-alpha) confidence region: The set
# of beta_1^0 with p-value >= alpha. 
pdf(file="Figures/condPbeta.pdf")
par(ps=18)
plot(betas,log10(pp),xlab=expression(beta[1]^0),
     ylab="log10 p-value",type="l")
# The p-values are increasing, so upper limit of confidence region 
# is infinity. For the lower limit, find the smallest beta_1^0 such 
# that the p-value is >= alpha:
conf.region.lower = min(betas[pp>=alpha]) # 1.462
# Draw a horizontal line at alpha and vertical line at
# conf.region.lower to illustrate the lower confidence limit.
abline(h=log10(alpha),lty=2) 
abline(v=conf.region.lower,lty=2)
dev.off()

# Though hard to see on the above plot, there is a dip in the 
# p-value curve around beta_1^0=1.48 or so.
# Zoom in to the area near beta_1^0=1.48 to show that the conf. region
# might not be an interval. By trial and error, the region of
# interest is from the 231st to 256th indices of the beta_1^0's.
inds=(231:256)
pdf(file="Figures/condPbetaZoom.pdf")
par(ps=18)
xlims = c(1.46,1.508); ylims=c(-1.234,-1.23)
plot(betas[inds],log10(pp[inds]),xlab=expression(beta[1]^0),
     ylab="log10 p-value",ylim=c(-1.234,-1.23),xlim=c(1.46,1.508),
     type="l")
# A test-level of .05854 would give a non-interval confidence region.
abline(h=log10(.05854),lty=2)
# Use graphical parameter "xpd" to add lines outside plotting 
# region to indicate the confidence region.
# These lines may not show up on all graphics devices but they 
# will show up in the PDF file.
par(xpd=TRUE) 
# There was a lot of trial-and-error to get these lines to look right
lines(c(1.4624,1.4743),rep(-1.2347,2),lwd=4)
lines(c(1.494,1.51),rep(-1.2347,2),lwd=4)
dev.off()

# For the other CI suggested by Mehta and Patel, obtained by  
# combining one-sided confidence limits, we need to calculate areas
# to the right and left of t1, under the conditional distribution 
# with different hypothesized beta_1^0 values.
# The arguments to the r.area and l.area functions are 
#  - cd, a conditional distribution object output by condDistT1()
#  - t1, the observed value of the sufficient statistic for the
#      parameter of interest
#  - beta10, the value of beta_1^0 for which we want the area
r.area = function(cd,t1,beta10) {
  if(t1 == min(cd$suppt)) {
    # All of the conditional distribution lies at or to the right of t1
    return(1) 
  } else {
    num = cd$counts*exp(cd$suppt*beta10)
    pmf = num/sum(num)
    p = sum(pmf[cd$suppt >= t1])
    return(p)
  }
}
l.area = function(cd,t1,beta10) {
  if(t1 == max(cd$suppt)) {
    # All of the conditional distribution lies at or to the left of t1
    return(1) 
  } else {
    num = cd$counts*exp(cd$suppt*beta10)
    pmf = num/sum(num)
    p = sum(pmf[cd$suppt <= t1])
    return(p)
  }
}

# Now plot right areas to read off the lower limit of the CI
# as the smallest beta_1^0 with p-value >= alpha/2.
gridlen = 1000
betas = seq(from=0,to=3,length=gridlen)
pp.r = rep(NA,gridlen)
for(i in 1:gridlen) {
  pp.r[i] = r.area(cd.DES,t1=7,beta10=betas[i])
}
pdf(file="Figures/rightTail.pdf")
par(ps=18)
plot(betas,log10(pp.r),xlab=expression(beta[1]^0),
     ylab="",type="l",cex=1)
title(ylab=expression(log[10](R(beta[1]^0~";"~7))),line=2)
abline(h=log10(alpha/2),lty=2)
# Lower confidence limit is the smallest beta_1^0 with 
# pp.r >= alpha/2
conf.int.lower = min(betas[pp.r>=alpha/2])
abline(v=conf.int.lower,lty=2)
dev.off()

# The observed t1 of 7 is the maximum of the support, so the areas 
# under the conditional pmf from t1 to the left are always 1; e.g., 
l.area(cd.DES,t1,beta10=1)
l.area(cd.DES,t1,beta10=3)
# In this case the upper limit of the confidence interval is infty.

#################################################################
## Point estimation

# Plot the conditional likelihood for the DES example.
# The clkhd function below takes the following arguments: 
#  - cd is a conditional distribution object, 
#  - t1 is the sufficient statistic for the parameter of interest
#  - beta1 is the value of beta_1 to calculate the cond. lkhd at
clkhd = function(cd,t1,beta1) {
  tem = cd$counts*exp(cd$suppt*beta1)
  num = tem[cd$suppt==t1] 
  return(num/sum(tem))
}
# Illustrate with a grid of 1000 points between beta_1=0 and 5.
gridlen = 1000
betas = seq(from=0,to=5,length=gridlen)
cc = rep(NA,gridlen)
for(i in 1:gridlen) {
  cc[i] = clkhd(cd.DES,t1,beta1=betas[i])
}
pdf(file="Figures/condLkhd.pdf")
par(ps=18)
plot(betas,log(cc),xlab=expression(beta[1]),
     ylab="log-conditional-likelihood",type="l")
dev.off()
# The conditional likelihood is increasing and so has no maximum;
# i.e., the CMLE doesn't exist.

# The MUE does exist though. In this case it is the smallest 
# beta_1^0 such that the area to the right of t1=7 is >= 1/2.
gridlen = 1000
betas = seq(from=2,to=4,length=gridlen)
pp.MUE = rep(NA,gridlen)
for(i in 1:gridlen) {
  pp.MUE[i] = r.area(cd.DES,t1,beta10=betas[i])
}
min(betas[pp.MUE >= 1/2]) # 3.19

#################################################################
### Limitations section
# Switch roles of DES and matern.smoking; i.e., make
# x1=matern.smoke and x2=DES. Find the conditional distribution
# of the sufficient statstic for the matern.smoke effect, given
# the sufficient statistic for DES.
# Use the function condDistT1() that we previously
# source()'d in from the file condDist.R.
cd.MS = condDistT1(x1=DES$matern.smoke,x2=DES$DES, d=DES$case,
                   matched.sets=DES$matched.set)
cd.MS

