rm(list=ls())
#---------------------------------------
# Simulations to draw density estimates of the 
# distribution of the Firth logistic regression estimator,
# beta^*, from simulations of data sets of 10 cases and 40 controls.
#---------------------------------------
# Load the logistf R package for its logistf() function
library(logistf)
#---------------------------------------
# Simulation function 
simdat = function(n0,n1,beta) {
  # n0 is number of controls and n1 of cases, beta is
  # regression effect of a single covariate x.
  # Dist'n of covariate x in controls is standard normal.
  # Can show it is normal with mean beta and sd 1 in cases.
  case = c(rep(1,n1),rep(0,n0))
  x = c(rnorm(n1,mean=beta,sd=1),rnorm(n0,mean=0,sd=1))
  return(data.frame(case=case,x=x))
}
#------------------------------------------------#
# Simulations for Figure 9.4.
n0 = 40; n1 = 10; form = formula(case ~ x)
# Simulate beta's of 0, 1, 2, and 3
bb = c(0:3)
dens = NULL # will hold density estimates of the beta^*'s
NSIM = 1000
set.seed(123)
ests = rep(NA,NSIM) # to record beta^* estimates
for(ii in 1:length(bb)){ # outer loop over beta's
  for(i in 1:NSIM){ # inner loop over simulation reps
    dat = simdat(n0,n1,beta=bb[ii])
    ests[i] = logistf(form,dat)$coef[2]
  }
  dens[[ii]] = density(ests) # record info for density estimate
}
# Now plot.
pdf(file="Figures/FirthEstrDistns.pdf")
plot(dens[[1]],xlab=expression(beta^{"*"}),ylab="density",
     main="",xlim=c(-1.5,5))
for(ii in 2:length(bb)) {
  lines(dens[[ii]],lty=ii)
}
legend("topright",legend=paste("beta=",(1:length(bb))-1,sep=""),
       lty=1:length(bb))
dev.off()

#---------------------------------------
# Figure 9.5 to compare HOA- and Firth-penalized profile
# likelhoods on the DES data
DES = read.csv("Data/DES.csv")
# Code for drawing HOA-penalized profile likelihood curves 
# for the log-odds ratio of interest is in the source file HOApenLkhd.R. 
# Code for drawing Firth-penalized profile likelihood curves 
# for the log-odds ratio of interest is in the source file FirthpenLkhd.R. 
source("Functions/HOApenLkhd.R")
source("Functions/FirthpenLkhd.R")
des = data.frame(case=DES$case,matchedset=DES$matched.set,DES=DES$DES)
firthcurve = FirthpenLkhd(des,beta1grid=seq(from=0,to=7,length=100))
pdf(file="Figures/HOAvsFirth.pdf")
hoacurve = HOApenLkhd(des,drawProfile=TRUE,beta1grid=seq(from=0,to=7,length=100)) 
abline(v=hoacurve$est[1])
lines(firthcurve$beta1grid,firthcurve$profileLkhd,lty=2)
require(logistf)
abline(v=firthcurve$beta1grid[which.max(firthcurve$profileLkhd)],lty=2)
dev.off()

#-------------------------------------------
# Table of estimates, standard errors and 95% confidence
# intervals for the DES effect, adjusted for maternal smoking, from 
# the different penalized profile likelihood approaches.
DES = read.csv("Data/DES.csv")
# Part I: unmatched analyses
source("Functions/logistzCC.R") # Zhang-penalized likelihood
library(logistf) # Firth-penalized
source("Functions/logf.R") # for log-F(1,1)- and log-F(2,2)-penalized likelihood
form = formula(case~DES+matern.smoke)
# Zhang; requires package nleqslv
library(nleqslv)
fit.Zhang = logistzCC(form,data=DES)
fit.Zhang$coef; fit.Zhang$ses
# Firth
fit.Firth = logistf(form,data=DES)
summary(fit.Firth)
# log-F(1,1)
ll = logF(form,DES,m=1) 
# NB: Call to logF warns of non-integer counts because of pseudo-response.
# This is normal.
summary(ll)
confint(ll,parm="DES") # more warnings of non-integer counts
# log-F(2,2)
ll = logF(form,DES,m=2)
summary(ll)
confint(ll,parm="DES")
#--------------------------------------------------
# Part II: matched analyses
form = formula(case~DES+matern.smoke)
# Firth conditional logistic regression is implemented by
# a wrapper function, clogitf(), that we wrote following the 
# instructions of G. Heinze. 
source("Functions/clogitf.R")
library(coxphf)
# clogitf() needs the matched set variable to be named "matcheset"
DES$matchedset = DES$matched.set 
fit = clogitf(form,DES,pl=TRUE)
summary(fit) # NB: CI is on OR scale. Take logs for log-OR scale
log(5.58179118); log(4150.4773)
# Compare to Firth logistic regression fit to the 
# stratified model. Need the matched set variable as 
# a factor so that logistf() will create the necessary 
# dummy variables for each matched set.
DES$fm = as.factor(DES$matched.set)
ff = logistf(case~fm + DES + matern.smoke,data=DES)
summary(ff)

